@@ -1,4 +1,5 @@
 arch_get_kernel_flavour () {
+	echo $MACHINE
 	return 0
 }
 
@@ -7,11 +8,12 @@ arch_check_usable_kernel () {
 }
 
 arch_get_kernel () {
-	version=2.4.27-2
+	imgbase=linux-image
+	version=$KERNEL_MAJOR-alpha
 	
 	if [ -n "$NUMCPUS" ] && [ "$NUMCPUS" -gt 1 ]; then
-		echo "kernel-image-$version-smp"
+		echo "$imgbase-$version-smp"
 	fi
 	
-	echo "kernel-image-$version-generic"
+	echo "$imgbase-$version-generic"
 }
