@@ -39,6 +39,7 @@ import debconf
 import apt_pkg
 from apt.cache import Cache
 from apt.progress import FetchProgress, InstallProgress
+from hashlib import md5
 
 sys.path.insert(0, '/usr/lib/ubiquity')
 
@@ -49,6 +50,16 @@ from ubiquity.components import language_apply, apt_setup, timezone_apply, \
                                 usersetup_apply, hw_detect, check_kernels, \
                                 migrationassistant_apply
 
+def debconf_disconnect():
+    """Disconnect from debconf. This is only to be used as a subprocess
+    preexec_fn helper."""
+    os.environ['DEBIAN_FRONTEND'] = 'noninteractive'
+    if 'DEBIAN_HAS_FRONTEND' in os.environ:
+        del os.environ['DEBIAN_HAS_FRONTEND']
+    if 'DEBCONF_USE_CDEBCONF' in os.environ:
+        # Probably not a good idea to use this in /target too ...
+        del os.environ['DEBCONF_USE_CDEBCONF']
+
 class DebconfFetchProgress(FetchProgress):
     """An object that reports apt's fetching progress using debconf."""
 
@@ -333,6 +344,12 @@ class Install:
                     # Exit code 3 signals to the frontend that we have
                     # handled this error.
                     sys.exit(3)
+                elif e.errno == errno.ENOSPC:
+                    error_template = 'ubiquity/install/copying_error/no_space'
+                    self.db.subst(error_template, 'ERROR', str(e))
+                    self.db.input('critical', error_template)
+                    self.db.go()
+                    sys.exit(3)
                 else:
                     raise
 
@@ -479,7 +496,11 @@ class Install:
         times = [(time_start, copied_size)]
         long_enough = False
         time_last_update = time_start
-
+        if self.db.get('ubiquity/install/md5_check') == 'false':
+            md5_check = False
+        else:
+            md5_check = True
+        
         old_umask = os.umask(0)
         for path in files:
             sourcepath = os.path.join(self.source, path)
@@ -506,9 +527,50 @@ class Install:
                     sourcefh = None
                     targetfh = None
                     try:
-                        sourcefh = open(sourcepath, 'rb')
-                        targetfh = open(targetpath, 'wb')
-                        shutil.copyfileobj(sourcefh, targetfh)
+                        while 1:
+                            sourcefh = open(sourcepath, 'rb')
+                            targetfh = open(targetpath, 'wb')
+                            if md5_check:
+                                sourcehash = md5()
+                            while 1:
+                                buf = sourcefh.read(16 * 1024)
+                                if not buf:
+                                    break
+                                targetfh.write(buf)
+                                if md5_check:
+                                    sourcehash.update(buf)
+
+                            if not md5_check:
+                                break
+                            targetfh.close()
+                            targetfh = open(targetpath, 'rb')
+                            if md5_check:
+                                targethash = md5()
+                            while 1:
+                                buf = targetfh.read(16 * 1024)
+                                if not buf:
+                                    break
+                                targethash.update(buf)
+                            if targethash.digest() != sourcehash.digest():
+                                if targetfh:
+                                    targetfh.close()
+                                if sourcefh:
+                                    sourcefh.close()
+                                error_template = 'ubiquity/install/copying_error/md5'
+                                self.db.subst(error_template, 'FILE', targetpath)
+                                self.db.input('critical', error_template)
+                                self.db.go()
+                                response = self.db.get(error_template)
+                                if response == 'skip':
+                                    break
+                                elif response == 'abort':
+                                    syslog.syslog(syslog.LOG_ERR,
+                                        'MD5 failure on %s' % targetpath)
+                                    sys.exit(3)
+                                elif response == 'retry':
+                                    pass
+                            else:
+                                break
                     finally:
                         if targetfh:
                             targetfh.close()
@@ -541,10 +603,11 @@ class Install:
                         times.pop(0)
                     speed = ((times[-1][1] - times[0][1]) /
                              (times[-1][0] - times[0][0]))
-                    time_remaining = int((total_size - copied_size) / speed)
-                    if time_remaining < 60:
-                        self.db.progress(
-                            'INFO', 'ubiquity/install/copying_minute')
+                    if speed != 0:
+                        time_remaining = int((total_size - copied_size) / speed)
+                        if time_remaining < 60:
+                            self.db.progress(
+                                'INFO', 'ubiquity/install/copying_minute')
 
         # Apply timestamps to all directories now that the items within them
         # have been copied.
@@ -622,10 +685,14 @@ class Install:
                 sysloopf.close()
                 if sysloopsize == '0':
                     devnull = open('/dev/null')
+                    if osextras.find_on_path('udevadm'):
+                        udevinfo_cmd = ['udevadm', 'info']
+                    else:
+                        udevinfo_cmd = ['udevinfo']
+                    udevinfo_cmd.extend(
+                        ['-q', 'name', '-p', os.path.join('/block', sysloop)])
                     udevinfo = subprocess.Popen(
-                        ['udevinfo', '-q', 'name',
-                         '-p', os.path.join('/block', sysloop)],
-                        stdout=subprocess.PIPE, stderr=devnull)
+                        udevinfo_cmd, stdout=subprocess.PIPE, stderr=devnull)
                     devbase = udevinfo.communicate()[0]
                     devnull.close()
                     if udevinfo.returncode != 0:
@@ -729,7 +796,7 @@ class Install:
                     "Failed to detach loopback device %s" % dev)
 
 
-    def chroot_setup(self):
+    def chroot_setup(self, x11=False):
         """Set up /target for safe package management operations."""
         policy_rc_d = os.path.join(self.target, 'usr/sbin/policy-rc.d')
         f = open(policy_rc_d, 'w')
@@ -756,8 +823,34 @@ exit 0"""
         if not os.path.exists(os.path.join(self.target, 'sys/devices')):
             self.chrex('mount', '-t', 'sysfs', 'sysfs', '/sys')
 
-    def chroot_cleanup(self):
+        if x11 and 'DISPLAY' in os.environ:
+            if 'SUDO_USER' in os.environ:
+                xauthority = os.path.expanduser('~%s/.Xauthority' %
+                                                os.environ['SUDO_USER'])
+            else:
+                xauthority = os.path.expanduser('~/.Xauthority')
+            if os.path.exists(xauthority):
+                shutil.copy(xauthority,
+                            os.path.join(self.target, 'root/.Xauthority'))
+
+            if not os.path.isdir(os.path.join(self.target, 'tmp/.X11-unix')):
+                os.mkdir(os.path.join(self.target, 'tmp/.X11-unix'))
+            misc.execute('mount', '--bind', '/tmp/.X11-unix',
+                         os.path.join(self.target, 'tmp/.X11-unix'))
+
+    def chroot_cleanup(self, x11=False):
         """Undo the work done by chroot_setup."""
+        if x11 and 'DISPLAY' in os.environ:
+            misc.execute('umount', os.path.join(self.target, 'tmp/.X11-unix'))
+            try:
+                os.rmdir(os.path.join(self.target, 'tmp/.X11-unix'))
+            except OSError:
+                pass
+            try:
+                os.unlink(os.path.join(self.target, 'root/.Xauthority'))
+            except OSError:
+                pass
+
         self.chrex('umount', '/sys')
         self.chrex('umount', '/proc')
 
@@ -802,7 +895,7 @@ exit 0"""
         # fontconfig configuration needs to be adjusted based on the
         # selected locale (from language-selector-common.postinst). Ignore
         # errors.
-        self.chrex('fontconfig-voodoo', '--auto', '--quiet')
+        self.chrex('fontconfig-voodoo', '--auto', '--force', '--quiet')
 
 
     def configure_apt(self):
@@ -862,7 +955,7 @@ exit 0"""
             }""")
         apt_conf_nmc.close()
 
-        dbfilter = apt_setup.AptSetup(None)
+        dbfilter = apt_setup.AptSetup(None, self.db)
         ret = dbfilter.run_command(auto_process=True)
         if ret != 0:
             raise InstallStepError("AptSetup failed with code %d" % ret)
@@ -1072,6 +1165,11 @@ exit 0"""
             os.unlink('/target/etc/usplash.conf')
         except OSError:
             pass
+        try:
+            modes = self.db.get('xserver-xorg/config/display/modes')
+            self.set_debconf('xserver-xorg/config/display/modes', modes)
+        except debconf.DebconfError:
+            pass
 
         try:
             os.unlink('/target/etc/popularity-contest.conf')
@@ -1087,13 +1185,24 @@ exit 0"""
             os.unlink('/target/etc/papersize')
         except OSError:
             pass
-        self.chrex('ucf', '--purge', '/etc/papersize')
+        subprocess.call(['log-output', '-t', 'ubiquity', 'chroot', self.target,
+                         'ucf', '--purge', '/etc/papersize'],
+                        preexec_fn=debconf_disconnect, close_fds=True)
         try:
             self.set_debconf('libpaper/defaultpaper', '')
         except debconf.DebconfError:
             pass
 
-        self.chroot_setup()
+        try:
+            os.unlink('/target/etc/ssl/certs/ssl-cert-snakeoil.pem')
+        except OSError:
+            pass
+        try:
+            os.unlink('/target/etc/ssl/private/ssl-cert-snakeoil.key')
+        except OSError:
+            pass
+
+        self.chroot_setup(x11=True)
         self.chrex('dpkg-divert', '--package', 'ubiquity', '--rename',
                    '--quiet', '--add', '/usr/sbin/update-initramfs')
         try:
@@ -1105,7 +1214,8 @@ exit 0"""
                     'linux-restricted-modules-' + self.kernel_version,
                     'usplash',
                     'popularity-contest',
-                    'libpaper1']
+                    'libpaper1',
+                    'ssl-cert']
 
         try:
             for package in packages:
@@ -1118,7 +1228,7 @@ exit 0"""
             self.chrex('dpkg-divert', '--package', 'ubiquity', '--rename',
                        '--quiet', '--remove', '/usr/sbin/update-initramfs')
             self.chrex('update-initramfs', '-c', '-k', os.uname()[2])
-            self.chroot_cleanup()
+            self.chroot_cleanup(x11=True)
 
         # Fix up kernel symlinks now that the initrd exists. Depending on
         # the architecture, these may be in / or in /boot.
@@ -1654,9 +1764,18 @@ exit 0"""
                         print >>kpersonalizerrc_file, 'FirstLogin=false'
                         kpersonalizerrc_file.close()
                         open('%s.created-by-oem', 'w').close()
+		# Carry the locale setting over to the installed system.
+		# This mimics the behavior in 01oem-config-udeb.
+                di_locale = self.db.get('debian-installer/locale')
+                if di_locale:
+                    self.set_debconf('debian-installer/locale', di_locale)
         except debconf.DebconfError:
             pass
 
+        # Tell apt-install to install packages directly from now on.
+        apt_install_direct = open('/var/lib/ubiquity/apt-install-direct', 'w')
+        apt_install_direct.close()
+
 
     def remove_extras(self):
         """Try to remove packages that are needed on the live CD but not on
@@ -1690,6 +1809,20 @@ exit 0"""
         if len(difference) == 0:
             return
 
+        use_restricted = True
+        try:
+            if self.db.get('apt-setup/restricted') == 'false':
+                use_restricted = False
+        except debconf.DebconfError:
+            pass
+        if not use_restricted:
+            cache = Cache()
+            for pkg in cache.keys():
+                if (cache[pkg].isInstalled and
+                    cache[pkg].section.startswith('restricted/')):
+                    difference.add(pkg)
+            del cache
+
         # Don't worry about failures removing packages; it will be easier
         # for the user to sort them out with a graphical package manager (or
         # whatever) after installation than it will be to try to deal with
@@ -1702,6 +1835,11 @@ exit 0"""
 
         misc.execute('umount', os.path.join(self.target, 'cdrom'))
 
+        env = dict(os.environ)
+        env['OVERRIDE_BASE_INSTALLABLE'] = '1'
+        subprocess.call(['/usr/lib/ubiquity/apt-setup/finish-install'],
+                        env=env)
+
         for apt_conf in ('00NoMountCDROM', '00IgnoreTimeConflict',
                          '00AllowUnauthenticated'):
             try:
@@ -1750,11 +1888,16 @@ exit 0"""
             dccomm.wait()
 
 
+    def reconfigure_preexec(self):
+        debconf_disconnect()
+        os.environ['XAUTHORITY'] = '/root/.Xauthority'
+
     def reconfigure(self, package):
         """executes a dpkg-reconfigure into installed system to each
         package which provided by args."""
-
-        self.chrex('dpkg-reconfigure', '-fnoninteractive', package)
+        subprocess.call(['log-output', '-t', 'ubiquity', 'chroot', self.target,
+                         'dpkg-reconfigure', '-fnoninteractive', package],
+                        preexec_fn=self.reconfigure_preexec, close_fds=True)
 
 
 if __name__ == '__main__':
