@@ -28,7 +28,7 @@ class GrubInstaller(FilteredCommand):
     def error(self, priority, question):
         self.frontend.error_dialog(self.description(question),
                                    self.extended_description(question))
-        return super(GrubInstaller, self).error(priority, question)
+        return FilteredCommand.error(self, priority, question)
 
     def run(self, priority, question):
         if question == 'grub-installer/bootdev':
@@ -36,4 +36,4 @@ class GrubInstaller(FilteredCommand):
             if self.db.get(question) == '':
                 self.preseed(question, '(hd0)')
 
-        return super(GrubInstaller, self).run(priority, question)
+        return FilteredCommand.run(self, priority, question)
