@@ -55,10 +55,10 @@ class MythbuntuPlugins(FilteredCommand):
         questions = ['^mythbuntu/mytharchive',
              '^mythbuntu/mythbrowser',
              '^mythbuntu/mythcontrols',
-             '^mythbuntu/mythdvd',
              '^mythbuntu/mythflix',
              '^mythbuntu/mythgallery',
              '^mythbuntu/mythgame',
+             '^mythbuntu/mythmovies',
              '^mythbuntu/mythmusic',
              '^mythbuntu/mythnews',
              '^mythbuntu/mythphone',
@@ -78,9 +78,6 @@ class MythbuntuPlugins(FilteredCommand):
         elif question.startswith('mythbuntu/mythcontrols'):
             mythcontrols = self.frontend.get_mythcontrols()
             self.preseed_bool('mythbuntu/mythcontrols', mythcontrols)
-        elif question.startswith('mythbuntu/mythdvd'):
-            mythdvd = self.frontend.get_mythdvd()
-            self.preseed_bool('mythbuntu/mythdvd', mythdvd)
         elif question.startswith('mythbuntu/mythflix'):
             mythflix = self.frontend.get_mythflix()
             self.preseed_bool('mythbuntu/mythflix', mythflix)
@@ -90,6 +87,9 @@ class MythbuntuPlugins(FilteredCommand):
         elif question.startswith('mythbuntu/mythgame'):
             mythgame = self.frontend.get_mythgame()
             self.preseed_bool('mythbuntu/mythgame', mythgame)
+        elif question.startswith('mythbuntu/mythmovies'):
+            mythmovies = self.frontend.get_mythmovies()
+            self.preseed_bool('mythbuntu/mythmovies', mythmovies)
         elif question.startswith('mythbuntu/mythmusic'):
             mythmusic = self.frontend.get_mythmusic()
             self.preseed_bool('mythbuntu/mythmusic', mythmusic)
@@ -120,14 +120,14 @@ class MythbuntuPlugins(FilteredCommand):
         self.preseed_bool('mythbuntu/mythbrowser', mythbrowser)
         mythcontrols = self.frontend.get_mythcontrols()
         self.preseed_bool('mythbuntu/mythcontrols', mythcontrols)
-        mythdvd = self.frontend.get_mythdvd()
-        self.preseed_bool('mythbuntu/mythdvd', mythdvd)
         mythflix = self.frontend.get_mythflix()
         self.preseed_bool('mythbuntu/mythflix', mythflix)
         mythgallery = self.frontend.get_mythgallery()
         self.preseed_bool('mythbuntu/mythgallery', mythgallery)
         mythgame = self.frontend.get_mythgame()
         self.preseed_bool('mythbuntu/mythgame', mythgame)
+        mythmovies = self.frontend.get_mythmovies()
+        self.preseed_bool('mythbuntu/mythmovies', mythmovies)
         mythmusic = self.frontend.get_mythmusic()
         self.preseed_bool('mythbuntu/mythmusic', mythmusic)
         mythnews = self.frontend.get_mythnews()
@@ -145,6 +145,8 @@ class MythbuntuPlugins(FilteredCommand):
         FilteredCommand.ok_handler(self)
 
 class MythbuntuThemes(FilteredCommand):
+#since all themes are pre-installed, we are seeding the ones
+#that will be *removed*
     def prepare(self):
         questions = ['^mythbuntu/officialthemes',
              '^mythbuntu/communitythemes']
@@ -153,18 +155,34 @@ class MythbuntuThemes(FilteredCommand):
     def run(self,priority,question):
         if question.startswith('mythbuntu/officialthemes'):
             official = self.frontend.get_officialthemes()
-            self.preseed_bool('mythbuntu/officialthemes', official)
+            official_string=""
+            for theme in official:
+                if not official[theme].get_active():
+                    official_string+=theme + " "
+            self.preseed('mythbuntu/officialthemes', official_string)
         elif question.startswith('mythbuntu/communitythemes'):
             community = self.frontend.get_communitythemes()
-            self.preseed_bool('mythbuntu/communitythemes', community)
+            community_string=""
+            for theme in community:
+                if not community[theme].get_active():
+                    community_string+=theme + " "
+            self.preseed('mythbuntu/communitythemes', community_string)
 
         return FilteredCommand.run(self, priority, question)
 
     def ok_handler(self):
         official = self.frontend.get_officialthemes()
-        self.preseed_bool('mythbuntu/officialthemes', official)
+        official_string=""
+        for theme in official:
+            if not official[theme].get_active():
+                official_string+=theme + " "
+        self.preseed('mythbuntu/officialthemes', official_string)
         community = self.frontend.get_communitythemes()
-        self.preseed_bool('mythbuntu/communitythemes', community)
+        community_string=""
+        for theme in community:
+            if not community[theme].get_active():
+                community_string+=theme + " "
+        self.preseed('mythbuntu/communitythemes', community_string)
         FilteredCommand.ok_handler(self)
 
 class MythbuntuServices(FilteredCommand):
@@ -311,66 +329,56 @@ def ok_handler(self):
 
 class MythbuntuRemote(FilteredCommand):
     def prepare(self):
-        questions = ['^mythbuntu/en_lirc',
-             '^lirc/remote',
-             '^lirc/lircd_conf',
-             '^lirc/modules',
-             '^lirc/driver']
-        return (['/usr/share/ubiquity/ask-remote'], questions)
+        questions = ['^lirc/remote',
+             '^lirc/remote_lircd_conf',
+             '^lirc/remote_modules',
+             '^lirc/remote_driver',
+             '^lirc/transmitter',
+             '^lirc/transmitter_lircd_conf',
+             '^lirc/transmitter_modules',
+             '^lirc/transmitter_driver']
+        return (['/usr/share/ubiquity/ask-ir'], questions)
 
     def run(self,priority,question):
-        if question.startswith('mythbuntu/en_lirc'):
-            en_lirc = self.frontend.get_lirc()
-            self.preseed_bool('mythbuntu/en_lirc',en_lirc)
-        elif question.startswith('lirc/remote'):
-            if self.frontend.get_lirc():
-                remote = self.frontend.get_lirc_remote()
-            else:
-                remote = "n/a"
-            self.preseed('lirc/remote',remote)
-        elif question.startswith('lirc/lircd_conf'):
-            if self.frontend.get_lirc():
-                config = self.frontend.get_lirc_rc()
-            else:
-                config = "n/a"
-            self.preseed('lirc/lircd_conf',config)
-        elif question.startswith('lirc/modules'):
-            if self.frontend.get_lirc():
-                modules = self.frontend.get_lirc_modules()
-            else:
-                modules = "n/a"
-            self.preseed('lirc/modules',modules)
-        elif question.startswith('lirc/driver'):
-            if self.frontend.get_lirc():
-                driver = self.frontend.get_lirc_driver()
-            else:
-                driver = "n/a"
-            self.preseed('lirc/driver',driver)
+        if question.startswith('lirc/remote'):
+            device=self.frontend.get_lirc("remote")
+            if question.startswith('lirc/remote_modules'):
+                self.preseed('lirc/remote_modules',device["modules"])
+            elif question.startswith('lirc/remote_lircd_conf'):
+                self.preseed('lirc/remote_lircd_conf',device["lircd_conf"])
+            elif question.startswith('lirc/remote_driver'):
+                self.preseed('lirc/remote_driver',device["driver"])
+            elif question.startswith('lirc/remote_device'):
+                self.preseed('lirc/remote_device',device["device"])
+            elif question.startswith('lirc/remote'):
+                self.preseed('lirc/remote',device["remote"])
+        elif question.startswith('lirc/transmitter'):
+            device=self.frontend.get_lirc("transmitter")
+            if question.startswith('lirc/transmitter_modules'):
+                self.preseed('lirc/transmitter_modules',device["modules"])
+            elif question.startswith('lirc/transmitter_lircd_conf'):
+                self.preseed('lirc/transmitter_lircd_conf',device["lircd_conf"])
+            elif question.startswith('lirc/transmitter_driver'):
+                self.preseed('lirc/transmitter_driver',device["driver"])
+            elif question.startswith('lirc/transmitter_device'):
+                self.preseed('lirc/transmitter_device',device["device"])
+            elif question.startswith('lirc/transmitter'):
+                self.preseed('lirc/transmitter',device["transmitter"])
         return FilteredCommand.run(self, priority, question)
 
     def ok_handler(self):
-        en_lirc = self.frontend.get_lirc()
-        self.preseed_bool('mythbuntu/en_lirc',en_lirc)
-        if self.frontend.get_lirc():
-            remote = self.frontend.get_lirc_remote()
-        else:
-            remote = "n/a"
-        self.preseed('lirc/remote',remote)
-        if self.frontend.get_lirc():
-            config = self.frontend.get_lirc_rc()
-        else:
-            config = "n/a"
-        self.preseed('lirc/lircd_conf',config)
-        if self.frontend.get_lirc():
-            modules = self.frontend.get_lirc_modules()
-        else:
-            modules = "n/a"
-        self.preseed('lirc/modules',modules)
-        if self.frontend.get_lirc():
-            driver = self.frontend.get_lirc_driver()
-        else:
-            driver = "n/a"
-        self.preseed('lirc/driver',driver)
+        device = self.frontend.get_lirc("remote")
+        self.preseed('lirc/remote_modules',device["modules"])
+        self.preseed('lirc/remote_lircd_conf',device["lircd_conf"])
+        self.preseed('lirc/remote_driver',device["driver"])
+        self.preseed('lirc/remote_device',device["device"])
+        self.preseed('lirc/remote',device["remote"])
+        device = self.frontend.get_lirc("transmitter")
+        self.preseed('lirc/transmitter_modules',device["modules"])
+        self.preseed('lirc/transmitter_lircd_conf',device["lircd_conf"])
+        self.preseed('lirc/transmitter_driver',device["driver"])
+        self.preseed('lirc/transmitter_device',device["device"])
+        self.preseed('lirc/transmitter',device["transmitter"])
         FilteredCommand.ok_handler(self)
 
 class MythbuntuDrivers(FilteredCommand):
@@ -379,7 +387,8 @@ class MythbuntuDrivers(FilteredCommand):
              '^mythbuntu/tvout',
              '^mythbuntu/tvstandard',
              '^mythbuntu/hdhomerun',
-             '^mythbuntu/xmltv']
+             '^mythbuntu/xmltv',
+             '^mythbuntu/dvbutils']
         return (['/usr/share/ubiquity/ask-drivers'], questions)
 
     def run(self,priority,question):
@@ -396,8 +405,11 @@ class MythbuntuDrivers(FilteredCommand):
             hdhomerun = self.frontend.get_hdhomerun()
             self.preseed_bool('mythbuntu/hdhomerun',hdhomerun)
         elif question.startswith('mythbuntu/xmltv'):
-            hdhomerun = self.frontend.get_xmltv()
+            xmltv = self.frontend.get_xmltv()
             self.preseed_bool('mythbuntu/xmltv',xmltv)
+        elif question.startswith('mythbuntu/dvbutils'):
+            dvbutils = self.frontend.get_dvbutils()
+            self.preseed_bool('mythbuntu/dvbutils',dvbutils)
         return FilteredCommand.run(self, priority, question)
 
     def ok_handler(self):
