@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2005 Junta de Andalucía
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd.
-# Copyright (C) 2007, Mario Limonciello, for Mythbuntu
+# Copyright (C) 2007-2008, Mario Limonciello, for Mythbuntu
 # Copyright (C) 2007, Jared Greenwald, for Mythbuntu
 #
 # Authors:
@@ -50,6 +50,12 @@ import signal
 import gtk
 import MySQLdb
 
+#Lirc support
+from mythbuntu_common.lirc import LircHandler
+
+#Theme support
+from mythbuntu_common.dictionaries import *
+
 from ubiquity.misc import *
 from ubiquity.components import console_setup, language, timezone, usersetup, \
                                 partman, partman_commit, \
@@ -71,10 +77,10 @@ BREADCRUMB_STEPS = {
     "mythbuntu_stepInstallType": 6,
     "mythbuntu_stepCustomInstallType": 7,
     "mythbuntu_stepPlugins": 8,
-    "mythbuntu_stepThemes": 9,
+    "tab_themes": 9,
     "mythbuntu_stepServices": 10,
     "mythbuntu_stepPasswords": 11,
-    "mythbuntu_stepLirc": 12,
+    "tab_remote_control": 12,
     "mythbuntu_stepDrivers": 13,
     "stepReady": 14,
     "mythbuntu_stepBackendSetup": 15
@@ -94,10 +100,10 @@ SUBPAGES = [
     "mythbuntu_stepInstallType",
     "mythbuntu_stepCustomInstallType",
     "mythbuntu_stepPlugins",
-    "mythbuntu_stepThemes",
+    "tab_themes",
     "mythbuntu_stepServices",
     "mythbuntu_stepPasswords",
-    "mythbuntu_stepLirc",
+    "tab_remote_control",
     "mythbuntu_stepDrivers",
     "stepReady",
     "mythbuntu_stepBackendSetup"
@@ -113,6 +119,7 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
     def __init__(self, distro):
         del os.environ['UBIQUITY_MIGRATION_ASSISTANT']
         ubiquity.frontend.gtk_ui.Wizard.__init__(self,distro)
+
         self.populate_lirc()
         self.populate_video()
         self.backup=False
@@ -341,6 +348,7 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
         self.installing = False
 
     def set_page(self, n):
+        self.run_automation_error_cmd()
         gtk_ui_pages  = ['Language', 'ConsoleSetup', 'Timezone', 'Partman', 'UserSetup', 'Summary', 'MigrationAssistant']
         found = False
         for item in gtk_ui_pages:
@@ -353,7 +361,7 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             if n == 'MythbuntuAdvancedType':
                 self.set_current_page(self.steps.page_num(self.mythbuntu_stepInstallType))
             elif n == 'MythbuntuRemote':
-                self.set_current_page(self.steps.page_num(self.mythbuntu_stepLirc))
+                self.set_current_page(self.steps.page_num(self.tab_remote_control))
             elif n == 'MythbuntuDrivers':
                 self.set_current_page(self.steps.page_num(self.mythbuntu_stepDrivers))
             if n == 'MythbuntuInstallType':
@@ -361,53 +369,62 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             elif n == 'MythbuntuPlugins':
                 self.set_current_page(self.steps.page_num(self.mythbuntu_stepPlugins))
             elif n == 'MythbuntuThemes':
-                self.set_current_page(self.steps.page_num(self.mythbuntu_stepThemes))
+                self.set_current_page(self.steps.page_num(self.tab_themes))
             elif n == 'MythbuntuPasswords':
                 self.set_current_page(self.steps.page_num(self.mythbuntu_stepPasswords))
+                installtype=self.get_installtype()
+                if installtype != "Master Backend/Frontend" and installtype != "Master Backend":
+                    self.allow_go_forward(False)
             elif n == 'MythbuntuServices':
                 self.set_current_page(self.steps.page_num(self.mythbuntu_stepServices))
 
 #Added Methods
     def populate_lirc(self):
-        """Fills the lirc pages with the appropriate data"""
-        #Note, this requires lirc >= 0.8.2-0ubuntu3
-        hwdb = open('/usr/share/lirc/lirc.hwdb').readlines()
-        hwdb.sort()
-        #Filter out uncessary lines
-        filter = "^\#|^\["
-        #Filter out the /dev/input/eventX remote
-        filter += "|http"
-        pattern = re.compile(filter)
-        for line in hwdb:
-            if pattern.search(line) is None:
-                list = string.split(line, ";")
-                if len(list) > 1:
-                    #Make sure we have a config file before including
-                    if list[4] != "":
-                        self.lirc_remote.append_text(list[0].translate(string.maketrans('',''),','))
-                        self.lirc_driver.append_text(list[1])
-                        self.lirc_modules.append_text(list[2])
-                        self.lirc_rc.append_text(list[4])
-        self.lirc_remote.append_text("Other Remote")
+            """Fills the lirc pages with the appropriate data"""
+            self.remote_count = 0
+            self.transmitter_count = 0
+            self.lirc=LircHandler()
+            for item in self.lirc.get_possible_devices("remote"):
+                self.remote_list.append_text(item)
+                self.remote_count = self.remote_count + 1
+            for item in self.lirc.get_possible_devices("transmitter"):
+                self.transmitter_list.append_text(item)
+                self.transmitter_count = self.transmitter_count + 1
+            self.remote_list.set_active(0)
+            self.transmitter_list.set_active(0)
 
     def populate_video(self):
         """Finds the currently active video driver"""
-        vid = open('/etc/X11/xorg.conf')
-        start_filter = re.compile("Section \"Device\"")
-        driver_filter = re.compile("Driver")
-        section=False
-        for line in vid:
-            if not section and start_filter.search(line):
-                section=True
-            elif section and driver_filter.search(line):
-                list = string.split(line, '"')
-                if len(list) > 1:
-                    self.video_driver.append_text("Open Source Driver: " + list[1])
-                    self.video_driver.set_active(5)
-                    self.tvoutstandard.set_active(0)
-                    self.tvouttype.set_active(0)
-                    break
-        vid.close()
+        #disable reading xorg.conf.  not really a good idea anymore
+        #with how empty it is as of Hardy
+        self.video_driver.append_text("Open Source Driver")
+        self.video_driver.set_active(5)
+        self.tvoutstandard.set_active(0)
+        self.tvouttype.set_active(0)
+
+        #vid = open('/etc/X11/xorg.conf')
+        #start_filter = re.compile("Section \"Device\"")
+        #driver_filter = re.compile("Driver")
+        #section=False
+        #for line in vid:
+        #   if not section and start_filter.search(line):
+        #        section=True
+        #    elif section and driver_filter.search(line):
+        #        list = string.split(line, '"')
+        #        if len(list) > 1:
+        #            self.video_driver.append_text("Open Source Driver: " + list[1])
+        #            self.video_driver.set_active(5)
+        #            self.tvoutstandard.set_active(0)
+        #            self.tvouttype.set_active(0)
+        #            break
+        #        else:
+        #            section = False
+        #if not section:
+        #    self.video_driver.append_text("Open Source Driver")
+        #    self.video_driver.set_active(5)
+        #    self.tvoutstandard.set_active(0)
+        #    self.tvouttype.set_active(0)
+        #vid.close()
 
     def allow_go_backward(self, allowed):
         self.back.set_sensitive(allowed and self.allowed_change_step)
@@ -458,7 +475,9 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
 
     def do_mythtv_setup(self,widget):
         """Spawn MythTV-Setup binary."""
+        os.seteuid(0)
         execute("/usr/share/ubiquity/mythbuntu-setup")
+        drop_privileges()
 
     def do_connection_test(self,widget):
         """Tests to make sure that the backend is accessible"""
@@ -474,8 +493,10 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             cursor.close()
             db.close()
             result = "Successful"
+            self.allow_go_forward(True)
         except:
             result = "Failure"
+            self.allow_go_forward(False)
         self.connection_results_label.show()
         self.connection_results.set_text(result)
 
@@ -493,6 +514,7 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             self.enablemysql.set_active(False)
 
         else:
+            self.theme_mythbuntu.set_sensitive(False)
             self.master_backend_expander.hide()
             self.mythweb_expander.show()
             self.mysql_server_expander.show()
@@ -553,10 +575,10 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             self.mytharchive.set_active(enable)
             self.mythbrowser.set_active(enable)
             self.mythcontrols.set_active(enable)
-            self.mythdvd.set_active(enable)
             self.mythflix.set_active(enable)
             self.mythgallery.set_active(enable)
             self.mythgame.set_active(enable)
+            self.mythmovies.set_active(enable)
             self.mythmusic.set_active(enable)
             self.mythnews.set_active(enable)
             self.mythphone.set_active(enable)
@@ -647,14 +669,61 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             set_fe_drivers(self,True)
             set_be_drivers(self,False)
 
-    def lirc_toggled(self,widget):
-        """Called when the checkbox to configure a remote is toggled"""
-        if (self.lirc_enable.get_active()):
-            self.lirc_middle_vbox.set_sensitive(True)
-            self.lirc_remote.set_active(0)
-        else:
-            self.lirc_middle_vbox.set_sensitive(False)
-            self.lirc_remote.set_active(0)
+    def toggle_ir(self,widget):
+        """Called whenever a request to enable/disable remote is called"""
+        if widget is not None:
+            #turn on/off IR remote
+            if widget.get_name() == 'remotecontrol':
+                self.remote_hbox.set_sensitive(widget.get_active())
+                self.generate_lircrc_checkbox.set_sensitive(widget.get_active())
+                if widget.get_active() and self.remote_list.get_active() == 0:
+                        self.remote_list.set_active(1)
+                else:
+                    self.remote_list.set_active(0)
+            #turn on/off IR transmitter
+            elif widget.get_name() == "transmittercontrol":
+                self.transmitter_hbox.set_sensitive(widget.get_active())
+                if widget.get_active():
+                    if self.transmitter_list.get_active() == 0:
+                        self.transmitter_list.set_active(1)
+                else:
+                    self.transmitter_list.set_active(0)
+            #if our selected remote itself changed
+            elif widget.get_name() == 'remote_list':
+                self.generate_lircrc_checkbox.set_active(True)
+                if self.remote_list.get_active() == 0:
+                    custom = False
+                    self.remotecontrol.set_active(False)
+                    self.generate_lircrc_checkbox.set_active(False)
+                elif self.remote_list.get_active_text() == "Custom":
+                    custom = True
+                else:
+                    custom = False
+                    self.remote_driver.set_text("")
+                    self.remote_modules.set_text("")
+                    self.remote_device.set_text("")
+                self.remote_driver_hbox.set_sensitive(custom)
+                self.remote_modules_hbox.set_sensitive(custom)
+                self.remote_device_hbox.set_sensitive(custom)
+                self.remote_configuration_hbox.set_sensitive(custom)
+                self.browse_remote_lircd_conf.set_filename("/usr/share/lirc/remotes")
+            #if our selected transmitter itself changed
+            elif widget.get_name() == 'transmitter_list':
+                if self.transmitter_list.get_active() == 0:
+                    custom = False
+                    self.transmittercontrol.set_active(False)
+                elif self.transmitter_list.get_active_text() == "Custom":
+                    custom = True
+                else:
+                    custom = False
+                    self.transmitter_driver.set_text("")
+                    self.transmitter_modules.set_text("")
+                    self.transmitter_device.set_text("")
+                self.transmitter_driver_hbox.set_sensitive(custom)
+                self.transmitter_modules_hbox.set_sensitive(custom)
+                self.transmitter_device_hbox.set_sensitive(custom)
+                self.transmitter_configuration_hbox.set_sensitive(custom)
+                self.browse_transmitter_lircd_conf.set_filename("/usr/share/lirc/transmitters")
 
     def mythweb_toggled(self,widget):
         """Called when the checkbox to install Mythweb is toggled"""
@@ -680,9 +749,41 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
     def uselivemysqlinfo_toggled(self,widget):
         """Called when the checkbox to copy live mysql information is pressed"""
         if (self.uselivemysqlinfo.get_active()):
-            self.master_backend_table.hide()
+            #disable modifying
+            self.master_backend_table.set_sensitive(False)
+            #read in mysql.txt to set the new defaults
+            try:
+                in_f=open("/etc/mythtv/mysql.txt")
+                for line in in_f:
+                    if re.compile("^DBHostName").search(line):
+                        text=string.split(string.split(line,"=")[1],'\n')[0]
+                        self.old_mysql_server=self.mysql_server.get_text()
+                        self.mysql_server.set_text(text)
+                    elif re.compile("^DBUserName").search(line):
+                        text=string.split(string.split(line,"=")[1],'\n')[0]
+                        self.old_mysql_user=self.mysql_user.get_text()
+                        self.mysql_user.set_text(text)
+                    elif re.compile("^DBName").search(line):
+                        text=string.split(string.split(line,"=")[1],'\n')[0]
+                        self.old_mysql_database=self.mysql_database.get_text()
+                        self.mysql_database.set_text(text)
+                    elif re.compile("^DBPassword").search(line):
+                        text=string.split(string.split(line,"=")[1],'\n')[0]
+                        self.old_mysql_password=self.mysql_password.get_text()
+                        self.mysql_password.set_text(text)
+                in_f.close()
+            except IOError:
+                #in case for some reason we are missing mysql.txt
+                self.usemysqlinfo.set_active(False)
         else:
-            self.master_backend_table.show()
+            self.allow_go_forward(False)
+            self.connection_results_label.hide()
+            self.connection_results.set_text("Please Test your connection to proceed")
+            self.master_backend_table.set_sensitive(True)
+            self.mysql_server.set_text(self.old_mysql_server)
+            self.mysql_user.set_text(self.old_mysql_user)
+            self.mysql_password.set_text(self.old_mysql_password)
+            self.mysql_database.set_text(self.old_mysql_database)
 
     def usemythwebpassword_toggled(self,widget):
         """Called when the checkbox to set a mythweb password is pressed"""
@@ -814,14 +915,13 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
                 self.tvouttype.set_active(0)
         elif (widget is not None and widget.get_name() == 'video_driver'):
             type = widget.get_active()
-            if (type == 0 or type == 1 or type == 2 or type == 3):
+            if (type == 0 or type == 1 or type == 2 or type == 3 or type == 4):
                 self.tvout_vbox.set_sensitive(True)
             else:
                 self.tvout_vbox.set_sensitive(False)
                 self.tvoutstandard.set_active(0)
                 self.tvouttype.set_active(0)
 
-
     def toggle_tv_out (self,widget):
         """Called when the tv-out type is toggled"""
         if (self.tvouttype.get_active() == 0):
@@ -842,32 +942,6 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
         elif (self.tvoutstandard.get_active() == 0):
             self.tvouttype.set_active(0)
 
-    def lirc_changed(self,widget):
-        """Called when anything remote related is changed"""
-        if (widget is not None and widget.get_name() == 'lirc_remote'):
-            #If the remote type isn't other, don't
-            #allow them to change any other settings
-            if (widget.get_active_text() == "Other Remote"):
-                self.lirc_driver.set_sensitive(True)
-                self.lirc_driver_label.set_sensitive(True)
-                self.lirc_modules.set_sensitive(True)
-                self.lirc_modules_label.set_sensitive(True)
-                self.lirc_rc.set_sensitive(True)
-                self.lirc_rc_label.set_sensitive(True)
-                self.lirc_bug_vbox.show()
-            else:
-                self.lirc_driver.set_sensitive(False)
-                self.lirc_driver_label.set_sensitive(False)
-                self.lirc_modules.set_sensitive(False)
-                self.lirc_modules_label.set_sensitive(False)
-                self.lirc_rc.set_sensitive(False)
-                self.lirc_rc_label.set_sensitive(False)
-                self.lirc_bug_vbox.hide()
-
-            self.lirc_driver.set_active(widget.get_active())
-            self.lirc_modules.set_active(widget.get_active())
-            self.lirc_rc.set_active(widget.get_active())
-
     def get_advanced(self):
         """Returns if this is an advanced install"""
         return self.custominstall.get_active()
@@ -906,9 +980,9 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
         else:
             return False
 
-    def get_mythdvd(self):
-        """Returns the status of the mythdvd plugin"""
-        if self.mythdvd.get_active():
+    def get_mythmovies(self):
+        """Returns the status of the mythmovies plugin"""
+        if self.mythmovies.get_active():
             return True
         else:
             return False
@@ -985,17 +1059,11 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
 
     def get_officialthemes(self):
         """Returns the status of the official themes"""
-        if self.officialthemes.get_active():
-            return True
-        else:
-            return False
+        return get_official_theme_dictionary(self)
 
     def get_communitythemes(self):
         """Returns the status of the community themes"""
-        if self.communitythemes.get_active():
-            return True
-        else:
-            return False
+        return get_community_theme_dictionary(self)
     def get_video(self):
         """Returns the status of the video graphics drivers"""
         if (self.modifyvideodriver.get_active()):
@@ -1009,7 +1077,7 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             elif driver == 3:
                 return "nvidia_new"
             elif driver == 4:
-                return "openchrome"
+                return "pvr_350"
             else:
                 return "None"
         else:
@@ -1020,14 +1088,14 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
         if (self.modifyvideodriver.get_active()):
             return self.tvouttype.get_active_text()
         else:
-            return "TV Out Disabled"
+            return "Disable TV-Out"
 
     def get_tvstandard(self):
         """Returns the status of the TV Standard type"""
         if (self.modifyvideodriver.get_active()):
             return self.tvoutstandard.get_active_text()
         else:
-            return "TV Out Disabled"
+            return "Disable TV-Out"
 
     def get_uselivemysqlinfo(self):
         if (self.uselivemysqlinfo.get_active()):
@@ -1100,26 +1168,45 @@ class Wizard(ubiquity.frontend.gtk_ui.Wizard):
             return True
         else:
             return False
-    def get_lirc(self):
-        if self.lirc_enable.get_active():
-            return True
-        else:
-            return False
 
-    def get_lirc_remote(self):
-        return self.lirc_remote.get_active_text()
-
-    def get_lirc_modules(self):
-        return self.lirc_modules.get_active_text()
-
-    def get_lirc_driver(self):
-        return self.lirc_driver.get_active_text()
-
-    def get_lirc_rc(self):
-        return self.lirc_rc.get_active_text()
+    def get_lirc(self,type):
+        item = {"modules":"","device":"","driver":"","lircd_conf":""}
+        if type == "remote":
+            item["remote"]=self.remote_list.get_active_text()
+            if item["remote"] == "Custom":
+                item["modules"]=self.remote_modules.get_text()
+                item["device"]=self.remote_device.get_text()
+                item["driver"]=self.remote_driver.get_text()
+                item["lircd_conf"]=self.browse_remote_lircd_conf.get_filename()
+        elif type == "transmitter":
+            item["transmitter"]=self.transmitter_list.get_active_text()
+            if item["transmitter"] == "Custom":
+                item["modules"]=self.transmitter_modules.get_text()
+                item["device"]=self.transmitter_device.get_text()
+                item["driver"]=self.transmitter_driver.get_text()
+                item["lircd_conf"]=self.browse_transmitter_lircd_conf.get_filename()
+        return item
 
     def get_hdhomerun(self):
         return self.hdhomerun.get_active()
 
-	def get_xmltv(self):
-		return self.xmltv.get_active()
+    def get_xmltv(self):
+        return self.xmltv.get_active()
+
+    def get_dvbutils(self):
+        return self.dvbutils.get_active()
+
+    def toggle_meta(self,widget):
+        """Called whenever a request to enable / disable all plugins"""
+        if widget is not None:
+            list = []
+            name = widget.get_name()
+            if (name == 'officialthemes'):
+                list = get_official_theme_dictionary(self)
+            elif (name == 'communitythemes'):
+                list = get_community_theme_dictionary(self)
+
+            toggle = widget.get_active()
+            for item in list:
+                if list[item].flags() & gtk.SENSITIVE:
+                    list[item].set_active(toggle)
